/*
 * Samples taken from http://radar.oreilly.com/2011/09/java7-features.html
 */
package cz.muni.fi.pb162.newinjava7;

import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 *
 * @author tomp
 */
public class NewInJava7 {

    // using underscores to separate thousands (since Java 7)
    private static final int million = 1_000_000;

    /**
     * Nothing new in main, just a launcher...
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        pathInfo();
        
        Map<String, List<String>> countries = createStringToListOfStrings();
        showStringSwitch("first-cz", countries);
        showStringSwitch("list-cz", countries);
        showStringSwitch("first-cz", countries);
        showStringSwitch("list-cz", countries);
        showStringSwitch("Czech Republic", countries);
        
        newTry();
        newMultiCatch("-1234");
    }

    private static void showStringSwitch(
            String action,
            Map<String, List<String>> countryToCities) {

        // switch can work on Strings since Java 7, too. 
        // based on String.equals:
        switch (action) {
            case "list-cz":
                list(countryToCities.get("Czech Republic"));
                break;
            case "list-sk":
                list(countryToCities.get("Slovakia"));
                break;
            case "first-cz":
                System.out.println(countryToCities.get("Czech Republic").get(0));
                break;
            case "first-sk":
                System.out.println(countryToCities.get("Slovakia").get(0));
                break;
            default:
                list(countryToCities.get(action));
                break;
        }
    }

    /**
     * To illustrate the diamond type parameter syntax new in Java 7
     *
     * @return a map mapping countries to lists of their cities
     */
    private static Map<String, List<String>> createStringToListOfStrings() {
        // this is the new Java 7 generics syntax: 
        // use the diamond instead of full type parameter:
        Map<String, List<String>> countryToCities = new TreeMap<>();

        List<String> czCities = Arrays.asList("Prague", "Brno", "Ostrava", "Pilsen");
        countryToCities.put("Czech Republic", czCities);

        List<String> skCities = Arrays.asList("Bratislava", "Košice", "Banská Bystrica");
        countryToCities.put("Slovakia", skCities);

        return countryToCities;
    }

    private static void newTry() {

        // streams opened in () will automatically be closed 
        // like if they were in the finally block
        // there can be multiple such resources 
        // (which must implement java.lang.AutoCloseable interface)

        try (FileOutputStream fos = new FileOutputStream("movies.txt");
                DataOutputStream dos = new DataOutputStream(fos)) {

            dos.writeUTF("Java 7 Block Buster");

        } catch (IOException e) {
            // log the exception 
            // the streams will automatically be closed
        }

    }

    private static void newMultiCatch(String input) {

        try {
            int result = Integer.parseInt(input);
            System.out.println(result);

        } catch (NumberFormatException e) {
            System.out.println(input + " has bad format for integer");

            // multiple exception types will be caught here:
        } catch (NullPointerException | IllegalArgumentException e) {
            System.out.println(input + " is either null or illegal");
        }
    }

    private static void pathInfo() {

        Path path = Paths.get("/Users/tomp/Desktop/Devel/pb162");

        System.out.println("Number of Nodes:" + path.getNameCount());

        System.out.println("File Name:" + path.getFileName());

        System.out.println("File Root:" + path.getRoot());

        System.out.println("File Parent:" + path.getParent());

    }

    private static void list(List<String> countries) {
        for (String country : countries) {
            System.out.print(country + " ");
        }
        System.out.println();
    }
}
